//****************************************************************************
// CPU pport data selector.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

//`define tracing

module cpupport(clk
             ,result,a,x,y,sp
             ,pc
             ,cpu_if,ifCycle,ifReady,iCount,ir
             ,carry,zero,interrupt,decimal,overflow,negative,irq,nmi_req,prfRead,cpu_rw_r
             //,cpu_dw_r
             ,pp_Address, pp_Dout//,EppDstb
             );

input clk;

input [3:0] iCount;
input [95:0] ir;
input [7:0] result,a,x,y,sp;
input [15:0] pc;
input [15:1] cpu_if;
input ifCycle,ifReady,carry,zero,interrupt,decimal,overflow,negative,irq,nmi_req,prfRead,cpu_rw_r;
//input [31:0] cpu_dw_r;

//input EppDstb;
input [5:0] pp_Address;
output [7:0] pp_Dout;

parameter H = 1'b1;
parameter L = 1'b0;

`ifdef tracing

reg ds, ds2, ds3;
reg [13:0] trace_a;
reg trace_rw;
reg [15:0] pc2, spa, xy, misc, addr;
wire [7:0] pc_out, spa_out, xy_out, misc_out, addr_out;
wire [15:0] pc_out_a, spa_out_a, xy_out_a, misc_out_a, addr_out_a;

traceram pctrace(clk, trace_rw, trace_a[13:1], trace_a, pc2, pc_out, pc_out_a);
traceram spatrace(clk, trace_rw, trace_a[13:1], trace_a, spa, spa_out);
traceram xytrace(clk, trace_rw, trace_a[13:1], trace_a, xy, xy_out);
traceram misctrace(clk, trace_rw, trace_a[13:1], trace_a, misc, misc_out);
traceram addrtrace(clk, trace_rw, trace_a[13:1], trace_a, addr, addr_out);

`endif

//#################################################          
//pport processing and 1248 state machine
//this provides access to internal cpu registers
//from the pport
//common registers and wires...
reg [7:0] pp_Dout;

always @(posedge clk)
  begin

`ifdef tracing
    ds2 <= EppDstb;
    ds3 <= ds2;
    ds <= ds2 & ~ds3;
`endif

    case(pp_Address)
      //CPU_STATE_LO
//      0 : pp_Dout <= { cpu_byte_enable_i, state };
      //ALU
      1 : pp_Dout <= result;
      //CPU_PC_LO
      2 : pp_Dout <= pc[7:0];
      //CPU_PC_HI
      3 : pp_Dout <= pc[15:8];
      //CPU_A
      4 : pp_Dout <= a;
      //CPU_X
      5 : pp_Dout <= x;
      //CPU_Y
      6 : pp_Dout <= y;
      //CPU_P
      7 :
        begin
          pp_Dout[0] <= carry;
          pp_Dout[1] <= zero;
          pp_Dout[2] <= interrupt;
          pp_Dout[3] <= decimal;
          pp_Dout[4] <= irq;
          pp_Dout[5] <= nmi_req;
          pp_Dout[6] <= overflow;
          pp_Dout[7] <= negative;
        end
      //CPU_SP
      8 : pp_Dout <= sp;
      //CPU_IF_LO
      9 : pp_Dout <= {cpu_if[7:1], L};
      //CPU_IF_HI
      10 : pp_Dout <= cpu_if[15:8];
      //CPU_IR_COUNT
      11 :
        begin
          pp_Dout[7] <= ifCycle;
          pp_Dout[6] <= ifReady;
          pp_Dout[5] <= prfRead;
          pp_Dout[4] <= cpu_rw_r;
          pp_Dout[3:0] <= iCount[3:0];
        end
      //CPU_IR_0
      12 : pp_Dout <= ir[7:0];
      //CPU_IR_1
      13 : pp_Dout <= ir[15:8];
      //CPU_IR_2
      14 : pp_Dout <= ir[23:16];
      //CPU_IR_3
      15 : pp_Dout <= ir[31:24];
      //CPU_IR_4
      16 : pp_Dout <= ir[39:32];
      //CPU_IR_5
      17 : pp_Dout <= ir[47:40];
      //CPU_IR_6
      18 : pp_Dout <= ir[55:48];
      //CPU_IR_7
      19 : pp_Dout <= ir[63:56];
      //CPU_IR_8
      20 : pp_Dout <= ir[71:64];
      //CPU_IR_9
      21 : pp_Dout <= ir[79:72];
      //CPU_IR_A
      22 : pp_Dout <= ir[87:80];
      //CPU_IR_B
      23 : pp_Dout <= ir[95:88];
//      //CPU_AB_LO
//      24 : pp_Dout <= cpu_a_debug[7:0];
//
//      //CPU_AB_HI
//      25 : pp_Dout <= cpu_a_debug[15:8];
//      //CPU_DW0
//      26 : pp_Dout <= cpu_dw_r[31:24];
//      //CPU_DW1
//      27 : pp_Dout <= cpu_dw_r[23:16];
//      //CPU_DW2
//      28 : pp_Dout <= cpu_dw_r[15:8];
//      //CPU_DW3
//      29 : pp_Dout <= cpu_dw_r[7:0];

`ifdef tracing
      //reset trace address register
      30 :  //(94)
        begin
          trace_a <= 0;
          pp_Dout <= 8'hDE;
        end
      //transfering pc trace data
      31 : //(95)
        begin
          pp_Dout <= pc_out;
          trace_rw <= H;
          
          if(ds)
            begin
              trace_a <= trace_a + H;
            end
        end
      //transfering sp a trace data
      32 : //(96)
        begin
          pp_Dout <= spa_out;
          trace_rw <= H;
          
          if(ds)
            begin
              trace_a <= trace_a + H;
            end
        end
      //transfering x y trace data
      33 : //(97)
        begin
          pp_Dout <= xy_out;
          trace_rw <= H;
          
          if(ds)
            begin
              trace_a <= trace_a + H;
            end
        end
      //transfering misc trace data
      34 : //(98)
        begin
          pp_Dout <= misc_out;
          trace_rw <= H;
          
          if(ds)
            begin
              trace_a <= trace_a + H;
            end
        end
      //gathering trace data
      35 : //(99)
        begin
          pp_Dout <= 8'hBE;
          
          if(pc == 16'hE55A)
            begin
              trace_a <= 14'h0000;
              trace_rw <= L;
              pc2 <= pc;
              spa <= { sp, a };
              xy <= { x, y };
              misc <= {state, cpu_wbyte, cpu_rw, prfRead, negative, overflow, decimal, interrupt, zero, carry };
              addr <= cpu_a;
            end
          else if(trace_a != 14'h3FFE)
            begin
              trace_a <= trace_a + 14'h0002;
              trace_rw <= L;
              pc2 <= pc;
              spa <= { sp, a };
              xy <= { x, y };
              misc <= {state, cpu_wbyte, cpu_rw, prfRead, negative, overflow, decimal, interrupt, zero, carry };
              addr <= cpu_a;
            end
          else
            begin
              trace_rw <= H;
            end
        end
      //transfering addr trace data
      36 : //(100)
        begin
          pp_Dout <= addr_out;
          trace_rw <= H;
          
          if(ds)
            begin
              trace_a <= trace_a + H;
            end
        end
`endif

      default :
        begin
          pp_Dout <= 8'h3C;
        end
    endcase
  end

endmodule


