//****************************************************************************
// CPU debug data display ROM.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

module debug_char(vclk, ain, dout, doutb);

input vclk;
input [11:0] ain;
output dout, doutb;

parameter H = 1'b1;
parameter L = 1'b0;
parameter c_0 = 6'h00;
parameter c_1 = 6'h01;
parameter c_2 = 6'h02;
parameter c_3 = 6'h03;
parameter c_4 = 6'h04;
parameter c_5 = 6'h05;
parameter c_6 = 6'h06;
parameter c_7 = 6'h07;
parameter c_8 = 6'h08;
parameter c_9 = 6'h09;
parameter c_A = 6'h0A;
parameter c_B = 6'h0B;
parameter c_C = 6'h0C;
parameter c_D = 6'h0D;
parameter c_E = 6'h0E;
parameter c_F = 6'h0F;
parameter c_G = 6'h10;
parameter c_H = 6'h11;
parameter c_I = 6'h12;
parameter c_J = 6'h13;
parameter c_K = 6'h14;
parameter c_L = 6'h15;
parameter c_M = 6'h16;
parameter c_N = 6'h17;
parameter c_O = 6'h18;
parameter c_P = 6'h19;
parameter c_Q = 6'h1A;
parameter c_R = 6'h1B;
parameter c_S = 6'h1C;
parameter c_T = 6'h1D;
parameter c_U = 6'h1E;
parameter c_V = 6'h1F;
parameter c_W = 6'h20;
parameter c_X = 6'h21;
parameter c_Y = 6'h22;
parameter c_Z = 6'h23;
parameter spc = 6'h24;
parameter neg = 6'h25;
parameter fwd = 6'h26;
parameter unk = 6'h27;

//this is a modified character rom for displaying debug information on the screen

   // BRAM_TDP_MACRO: True Dual Port RAM
   //                 Spartan-6
   // Xilinx HDL Language Template, version 14.4
   
   //////////////////////////////////////////////////////////////////////////
   // DATA_WIDTH_A/B | BRAM_SIZE | RAM Depth | ADDRA/B Width | WEA/B Width //
   // ===============|===========|===========|===============|=============//
   //     19-36      |  "18Kb"   |     512   |     9-bit     |    4-bit    //
   //     10-18      |  "18Kb"   |    1024   |    10-bit     |    2-bit    //
   //     10-18      |   "9Kb"   |     512   |     9-bit     |    2-bit    //
   //      5-9       |  "18Kb"   |    2048   |    11-bit     |    1-bit    //
   //      5-9       |   "9Kb"   |    1024   |    10-bit     |    1-bit    //
   //      3-4       |  "18Kb"   |    4096   |    12-bit     |    1-bit    //
   //      3-4       |   "9Kb"   |    2048   |    11-bit     |    1-bit    //
   //        2       |  "18Kb"   |    8192   |    13-bit     |    1-bit    //
   //        2       |   "9Kb"   |    4096   |    12-bit     |    1-bit    //
   //        1       |  "18Kb"   |   16384   |    14-bit     |    1-bit    //
   //        1       |   "9Kb"   |    8192   |    12-bit     |    1-bit    //
   //////////////////////////////////////////////////////////////////////////

   BRAM_TDP_MACRO #(
      .BRAM_SIZE("18Kb"), // Target BRAM: "9Kb" or "18Kb" 
      .DEVICE("SPARTAN6"), // Target device: "VIRTEX5", "VIRTEX6", "SPARTAN6" 
      .DOA_REG(1),        // Optional port A output register (0 or 1)
      .DOB_REG(1),        // Optional port B output register (0 or 1)
      .INIT_A(36'h0000000),  // Initial values on port A output port
      .INIT_B(36'h00000000), // Initial values on port B output port
      .INIT_FILE ("NONE"),
      .READ_WIDTH_A (1),   // Valid values are 1-36
      .READ_WIDTH_B (1),   // Valid values are 1-36
      .SIM_COLLISION_CHECK ("ALL"), // Collision check enable "ALL", "WARNING_ONLY", 
                                    //   "GENERATE_X_ONLY" or "NONE" 
      .SRVAL_A(36'h00000000), // Set/Reset value for port A output
      .SRVAL_B(36'h00000000), // Set/Reset value for port B output
      .WRITE_MODE_A("WRITE_FIRST"), // "WRITE_FIRST", "READ_FIRST", or "NO_CHANGE" 
      .WRITE_MODE_B("WRITE_FIRST"), // "WRITE_FIRST", "READ_FIRST", or "NO_CHANGE" 
      .WRITE_WIDTH_A(1), // Valid values are 1-36
      .WRITE_WIDTH_B(1), // Valid values are 1-36

//@
//A0 : 3C
//A1 : 66
//A2 : 76
//A3 : 76
//A4 : 06
//A5 : 46
//A6 : 3C
//A7 : 00
//A
//A8 : 18
//A9 : 3C
//A10 : 66
//A11 : 7E
//A12 : 66
//A13 : 66
//A14 : 66
//A15 : 00
//B
//A16 : 3E
//A17 : 66
//A18 : 66
//A19 : 3E
//A20 : 66
//A21 : 66
//A22 : 3E
//A23 : 00
//C
//A24 : 3C
//A25 : 66
//A26 : 06
//A27 : 06
//A28 : 06
//A29 : 66
//A30 : 3C
//A31 : 00
//D
//A32 : 1E
//A33 : 36
//A34 : 66
//A35 : 66
//A36 : 66
//A37 : 36
//A38 : 1E
//A39 : 00
//E
//A40 : 7E
//A41 : 06
//A42 : 06
//A43 : 1E
//A44 : 06
//A45 : 06
//A46 : 7E
//A47 : 00
//F
//A48 : 7E
//A49 : 06
//A50 : 06
//A51 : 1E
//A52 : 06
//A53 : 06
//A54 : 06
//A55 : 00
//G
//A56 : 3C
//A57 : 66
//A58 : 06
//A59 : 76
//A60 : 66
//A61 : 66
//A62 : 3C
//A63 : 00
//H
//A64 : 66
//A65 : 66
//A66 : 66
//A67 : 7E
//A68 : 66
//A69 : 66
//A70 : 66
//A71 : 00
//I
//A72 : 3C
//A73 : 18
//A74 : 18
//A75 : 18
//A76 : 18
//A77 : 18
//A78 : 3C
//A79 : 00
//J
//A80 : 78
//A81 : 30
//A82 : 30
//A83 : 30
//A84 : 30
//A85 : 36
//A86 : 1C
//A87 : 00
//K
//A88 : 66
//A89 : 36
//A90 : 1E
//A91 : 0E
//A92 : 1E
//A93 : 36
//A94 : 66
//A95 : 00
//L
//A96 : 06
//A97 : 06
//A98 : 06
//A99 : 06
//A100 : 06
//A101 : 06
//A102 : 7E
//A103 : 00
//M
//A104 : C6
//A105 : EE
//A106 : FE
//A107 : D6
//A108 : C6
//A109 : C6
//A110 : C6
//A111 : 00
//N
//A112 : 66
//A113 : 6E
//A114 : 7E
//A115 : 7E
//A116 : 76
//A117 : 66
//A118 : 66
//A119 : 00
//O
//A120 : 3C
//A121 : 66
//A122 : 66
//A123 : 66
//A124 : 66
//A125 : 66
//A126 : 3C
//A127 : 00
//P
//A128 : 3E
//A129 : 66
//A130 : 66
//A131 : 3E
//A132 : 06
//A133 : 06
//A134 : 06
//A135 : 00
//Q
//A136 : 3C
//A137 : 66
//A138 : 66
//A139 : 66
//A140 : 66
//A141 : 3C
//A142 : 70
//A143 : 00
//R
//A144 : 3E
//A145 : 66
//A146 : 66
//A147 : 3E
//A148 : 1E
//A149 : 36
//A150 : 66
//A151 : 00
//S
//A152 : 3C
//A153 : 66
//A154 : 06
//A155 : 3C
//A156 : 60
//A157 : 66
//A158 : 3C
//A159 : 00
//T
//A160 : 7E
//A161 : 18
//A162 : 18
//A163 : 18
//A164 : 18
//A165 : 18
//A166 : 18
//A167 : 00
//U
//A168 : 66
//A169 : 66
//A170 : 66
//A171 : 66
//A172 : 66
//A173 : 66
//A174 : 3C
//A175 : 00
//V
//A176 : 66
//A177 : 66
//A178 : 66
//A179 : 66
//A180 : 66
//A181 : 3C
//A182 : 18
//A183 : 00
//W
//A184 : C6
//A185 : C6
//A186 : C6
//A187 : D6
//A188 : FE
//A189 : EE
//A190 : C6
//A191 : 00
//X
//A192 : 66
//A193 : 66
//A194 : 3C
//A195 : 18
//A196 : 3C
//A197 : 66
//A198 : 66
//A199 : 00
//Y
//A200 : 66
//A201 : 66
//A202 : 66
//A203 : 3C
//A204 : 18
//A205 : 18
//A206 : 18
//A207 : 00
//Z
//A208 : 7E
//A209 : 60
//A210 : 30
//A211 : 18
//A212 : 0C
//A213 : 06
//A214 : 7E
//A215 : 00
//[
//A216 : 3C
//A217 : 0C
//A218 : 0C
//A219 : 0C
//A220 : 0C
//A221 : 0C
//A222 : 3C
//A223 : 00
//???
//A224 : 30
//A225 : 48
//A226 : 0C
//A227 : 3E
//A228 : 0C
//A229 : 46
//A230 : 3F
//A231 : 00
//]
//A232 : 3C
//A233 : 30
//A234 : 30
//A235 : 30
//A236 : 30
//A237 : 30
//A238 : 3C
//A239 : 00
//???
//A240 : 00
//A241 : 18
//A242 : 3C
//A243 : 7E
//A244 : 18
//A245 : 18
//A246 : 18
//A247 : 18
//???
//A248 : 00
//A249 : 08
//A250 : 0C
//A251 : FE
//A252 : FE
//A253 : 0C
//A254 : 08
//A255 : 00
//<sp>
//A256 : 00
//A257 : 00
//A258 : 00
//A259 : 00
//A260 : 00
//A261 : 00
//A262 : 00
//A263 : 00
//!
//A264 : 18
//A265 : 18
//A266 : 18
//A267 : 18
//A268 : 00
//A269 : 00
//A270 : 18
//A271 : 00
//"
//A272 : 66
//A273 : 66
//A274 : 66
//A275 : 00
//A276 : 00
//A277 : 00
//A278 : 00
//A279 : 00
//#
//A280 : 66
//A281 : 66
//A282 : FF
//A283 : 66
//A284 : FF
//A285 : 66
//A286 : 66
//A287 : 00
//$
//A288 : 18
//A289 : 7C
//A290 : 06
//A291 : 3C
//A292 : 60
//A293 : 3E
//A294 : 18
//A295 : 00
//%
//A296 : 46
//A297 : 66
//A298 : 30
//A299 : 18
//A300 : 0C
//A301 : 66
//A302 : 62
//A303 : 00
//&
//A304 : 3C
//A305 : 66
//A306 : 3C
//A307 : 1C
//A308 : E6
//A309 : 66
//A310 : FC
//A311 : 00
//'
//A312 : 60
//A313 : 30
//A314 : 18
//A315 : 00
//A316 : 00
//A317 : 00
//A318 : 00
//A319 : 00
//{
//A320 : 30
//A321 : 18
//A322 : 0C
//A323 : 0C
//A324 : 0C
//A325 : 18
//A326 : 30
//A327 : 00
//}
//A328 : 0C
//A329 : 18
//A330 : 30
//A331 : 30
//A332 : 30
//A333 : 18
//A334 : 0C
//A335 : 00
//*
//A336 : 00
//A337 : 66
//A338 : 3C
//A339 : FF
//A340 : 3C
//A341 : 66
//A342 : 00
//A343 : 00
//+
//A344 : 00
//A345 : 18
//A346 : 18
//A347 : 7E
//A348 : 18
//A349 : 18
//A350 : 00
//A351 : 00
//,
//A352 : 00
//A353 : 00
//A354 : 00
//A355 : 00
//A356 : 00
//A357 : 18
//A358 : 18
//A359 : 0C
//-
//A360 : 00
//A361 : 00
//A362 : 00
//A363 : 7E
//A364 : 00
//A365 : 00
//A366 : 00
//A367 : 00
//.
//A368 : 00
//A369 : 00
//A370 : 00
//A371 : 00
//A372 : 00
//A373 : 18
//A374 : 18
//A375 : 00
///
//A376 : 00
//A377 : C0
//A378 : 60
//A379 : 30
//A380 : 18
//A381 : 0C
//A382 : 06
//A383 : 00
//0
//A384 : 3C
//A385 : 66
//A386 : 76
//A387 : 6E
//A388 : 66
//A389 : 66
//A390 : 3C
//A391 : 00
//1
//A392 : 18
//A393 : 18
//A394 : 1C
//A395 : 18
//A396 : 18
//A397 : 18
//A398 : 7E
//A399 : 00
//2
//A400 : 3C
//A401 : 66
//A402 : 60
//A403 : 30
//A404 : 0C
//A405 : 06
//A406 : 7E
//A407 : 00
//3
//A408 : 3C
//A409 : 66
//A410 : 60
//A411 : 38
//A412 : 60
//A413 : 66
//A414 : 3C
//A415 : 00
//4
//A416 : 60
//A417 : 70
//A418 : 78
//A419 : 66
//A420 : FE
//A421 : 60
//A422 : 60
//A423 : 00
//5
//A424 : 7E
//A425 : 06
//A426 : 3E
//A427 : 60
//A428 : 60
//A429 : 66
//A430 : 3C
//A431 : 00
//6
//A432 : 3C
//A433 : 66
//A434 : 06
//A435 : 3E
//A436 : 66
//A437 : 66
//A438 : 3C
//A439 : 00
//7
//A440 : 7E
//A441 : 66
//A442 : 30
//A443 : 18
//A444 : 18
//A445 : 18
//A446 : 18
//A447 : 00
//8
//A448 : 3C
//A449 : 66
//A450 : 66
//A451 : 3C
//A452 : 66
//A453 : 66
//A454 : 3C
//A455 : 00
//9
//A456 : 3C
//A457 : 66
//A458 : 66
//A459 : 7C
//A460 : 60
//A461 : 66
//A462 : 3C
//A463 : 00
//:
//A464 : 00
//A465 : 00
//A466 : 18
//A467 : 00
//A468 : 00
//A469 : 18
//A470 : 00
//A471 : 00
//;
//A472 : 00
//A473 : 00
//A474 : 18
//A475 : 00
//A476 : 00
//A477 : 18
//A478 : 18
//A479 : 0C
//<
//A480 : 70
//A481 : 18
//A482 : 0C
//A483 : 06
//A484 : 0C
//A485 : 18
//A486 : 70
//A487 : 00
//=
//A488 : 00
//A489 : 00
//A490 : 7E
//A491 : 00
//A492 : 7E
//A493 : 00
//A494 : 00
//A495 : 00
//>
//A496 : 0E
//A497 : 18
//A498 : 30
//A499 : 60
//A500 : 30
//A501 : 18
//A502 : 0E
//A503 : 00
//?
//A504 : 3C
//A505 : 66
//A506 : 60
//A507 : 30
//A508 : 18
//A509 : 00
//A510 : 18
//A511 : 00

//4
//A416 : 60
//A417 : 70
//A418 : 78
//A419 : 66
//A420 : FE
//A421 : 60
//A422 : 60
//A423 : 00

      .INIT_00(256'h003C66603860663C007E060C3060663C007E1818181C1818003C66666E76663C),//0123
      .INIT_01(256'h001818181830667E003C66663E06663C003C6660603E067E006060FE666C7860),//4567
      .INIT_02(256'h003E66663E66663E006666667E663C18003C66607C66663C003C66663C66663C),//89AB
      .INIT_03(256'h000606061E06067E007E06061E06067E001E36666666361E003C66060606663C),//CDEF
      .INIT_04(256'h001C363030303078003C18181818183C006666667E666666003C66667606663C),//GHIJ
      .INIT_05(256'h006666767E7E6E6600C6C6C6D6FEEEC6007E0606060606060066361E0E1E3666),//KLMN
      .INIT_06(256'h0066361E3E66663E00703C666666663C000606063E66663E003C66666666663C),//OPQR
      .INIT_07(256'h00183C6666666666003C666666666666001818181818187E003C66603C06663C),//STUV
      .INIT_08(256'h007E060C1830607E001818183C6666660066663C183C666600C6EEFED6C6C6C6),//WXYZ
      .INIT_09(256'h000000000000000000060C183060C000000000007E0000000000000000000000),//<sp>-/
      .INIT_0A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0F(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_10(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_11(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_12(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_13(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_14(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_15(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_16(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_17(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_18(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_19(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1F(256'h0000000000000000000000000000000000000000000000000000000000000000),
            
      // The next set of INIT_xx are for "18Kb" configuration only
      .INIT_20(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_21(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_22(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_23(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_24(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_25(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_26(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_27(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_28(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_29(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2F(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_30(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_31(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_32(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_33(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_34(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_35(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_36(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_37(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_38(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_39(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3F(256'h0000000000000000000000000000000000000000000000000000000000000000),
     
      // The next set of INITP_xx are for the parity bits
      .INITP_00(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_01(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_02(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_03(256'h0000000000000000000000000000000000000000000000000000000000000000),
            
      // The next set of INITP_xx are for "18Kb" configuration only
      .INITP_04(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_05(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_06(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_07(256'h0000000000000000000000000000000000000000000000000000000000000000)
   ) BRAM_TDP_MACRO_inst (
      .DOA(dout),       // Output port-A data, width defined by READ_WIDTH_A parameter
      .DOB(doutb),      // Output port-B data, width defined by READ_WIDTH_B parameter
      .ADDRA({ 2'h0, ain[11:0] }),   // Input port-A address, width defined by Port A depth
      .ADDRB({ 2'h0, ain[11:0] }),   // Input port-B address, width defined by Port B depth
      .CLKA(vclk),     // 1-bit input port-A clock
      .CLKB(vclk),     // 1-bit input port-B clock
      .DIA(1'b0),       // Input port-A data, width defined by WRITE_WIDTH_A parameter
      .DIB(1'b0),       // Input port-B data, width defined by WRITE_WIDTH_B parameter
      .ENA(1'b1),       // 1-bit input port-A enable
      .ENB(1'b1),       // 1-bit input port-B enable
      .REGCEA(1'b1), // 1-bit input port-A output register enable
      .REGCEB(1'b1), // 1-bit input port-B output register enable
      .RSTA(1'b0),     // 1-bit input port-A reset
      .RSTB(1'b0),     // 1-bit input port-B reset
      .WEA(1'b0),       // Input port-A write enable, width defined by Port A depth
      .WEB(1'b0)        // Input port-B write enable, width defined by Port B depth
   );

   // End of BRAM_TDP_MACRO_inst instantiation

endmodule

