//****************************************************************************
// 6502 instruction decoding.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

module iDecode(clk, ir,
               two_byte, one_byte_a, three_byte_a,
               asla, rola, lsra, rora, dex, dey, inx, iny,txa, tya, txs, tay,
               tax, tsx, pha, pla,clc, sec, cli, sei, cld, sed, clv, php, plp,
               ora_i, anda_i, eora_i, lda_i, cmpa_i, adca_i, sbca_i,
               ora_z, anda_z, eora_z, lda_z, cmpa_z, adca_z, sbca_z, bita_z,
               ora_zx, anda_zx, eora_zx, lda_zx, cmpa_zx, adca_zx, sbca_zx,
               sta_ix, ora_ix, anda_ix, eora_ix, lda_ix, cmpa_ix, adca_ix, sbca_ix,
               sta_iy, ora_iy, anda_iy, eora_iy, lda_iy, cmpa_iy, adca_iy, sbca_iy,
               ora_a, anda_a, eora_a, lda_a, cmpa_a, bita_a, adca_a, sbca_a,
               ora_ax, anda_ax, eora_ax, ldy_ax, lda_ax, cmpa_ax, adca_ax, sbca_ax,
               ora_ay, anda_ay, eora_ay, lda_ay, ldx_ay, cmpa_ay, adca_ay, sbca_ay,
               asl_z, rol_z, lsr_z, ror_z, dec_z, inc_z,
               asl_zx, rol_zx, lsr_zx, ror_zx, dec_zx, inc_zx,
               asl_a, rol_a, lsr_a, ror_a, dec_a, inc_a,
               asl_ax, rol_ax, lsr_ax, ror_ax, dec_ax, inc_ax,
               stx_z, ldx_z, cpx_z, stx_zy, ldx_zy, stx_a, ldx_a, cpx_a, ldx_i, cpx_i,
               sty_z, ldy_z, cpy_z, sty_zx, ldy_zx, sty_a, ldy_a, cpy_a, ldy_i, cpy_i,
               bpl, bmi, bvc, bvs, bcc, bcs, bne, beq,
               brk, rti, rts, jmp_a, jmp_i, jsr_a,
               ix, iy, st_zxy, st_axy, zx, zy, ax, ay, mem_op, reg_op);// irr, nop, sta_z, sta_zx, sta_a, sta_ax, sta_ay);

input clk;
input [7:0] ir;
//output [7:0] irr;

output one_byte_a, three_byte_a;
output ix, iy, st_zxy, st_axy, zx, zy, ax, ay, mem_op, reg_op;
output two_byte;
output asla, rola, lsra, rora, dex, dey, inx, iny;
output txa, tya, txs, tay, tax, tsx, pha, pla;
output clc, sec, cli, sei, cld, sed, clv, php, plp;
output ora_i, anda_i, eora_i, lda_i, cmpa_i, adca_i, sbca_i;
output ora_z, anda_z, eora_z, lda_z, cmpa_z, adca_z, sbca_z, bita_z;
output ora_zx, anda_zx, eora_zx, lda_zx, cmpa_zx, adca_zx, sbca_zx;
output sta_ix, ora_ix, anda_ix, eora_ix, lda_ix, cmpa_ix, adca_ix, sbca_ix;
output sta_iy, ora_iy, anda_iy, eora_iy, lda_iy, cmpa_iy, adca_iy, sbca_iy;
output ora_a, anda_a, eora_a, lda_a, cmpa_a, bita_a, adca_a, sbca_a;
output ora_ax, anda_ax, eora_ax, ldy_ax, lda_ax, cmpa_ax, adca_ax, sbca_ax;
output ora_ay, anda_ay, eora_ay, lda_ay, ldx_ay, cmpa_ay, adca_ay, sbca_ay;
output asl_z, rol_z, lsr_z, ror_z, dec_z, inc_z;
output asl_zx, rol_zx, lsr_zx, ror_zx, dec_zx, inc_zx;
output asl_a, rol_a, lsr_a, ror_a, dec_a, inc_a;
output asl_ax, rol_ax, lsr_ax, ror_ax, dec_ax, inc_ax;
output stx_z, ldx_z, cpx_z, stx_zy, ldx_zy, stx_a, ldx_a, cpx_a, ldx_i, cpx_i;
output sty_z, ldy_z, cpy_z, sty_zx, ldy_zx, sty_a, ldy_a, cpy_a, ldy_i, cpy_i;
output bpl, bmi, bvc, bvs, bcc, bcs, bne, beq;
output brk, rti, rts, jmp_a, jmp_i, jsr_a;//nop, sta_z, sta_zx, sta_a, sta_ax, sta_ay;

parameter H = 1'b1;
parameter L = 1'b0;

//instruction set decode - 6510 cycles / X6510 cycles
//                         +cross page / +Decimal mode /* zp or stack page accessed with an absolute addressing mode
//,nmi, irq  7 + completion of current instruction? / 5 + completion of previous instruction

//program flow control
parameter BRK   = 8'h00; // 7 / 5 (a cached vector would reduce this by 2 more cycles,including irq/nmi)
parameter RTI   = 8'h40; // 6 / 4
parameter JSR_A = 8'h20; // 6 / 3
parameter RTS   = 8'h60; // 6 / 5
parameter JMP_A = 8'h4C; // 3 / 3
parameter JMP_I = 8'h6C; // 5 / 5

//conditional branches
parameter BPL = 8'h10; // 3+,2 / 3,1 (TAKEN,NOT TAKEN)
parameter BMI = 8'h30; // 3+,2 / 3,1
parameter BVC = 8'h50; // 3+,2 / 3,1
parameter BVS = 8'h70; // 3+,2 / 3,1
parameter BCC = 8'h90; // 3+,2 / 3,1
parameter BCS = 8'hB0; // 3+,2 / 3,1
parameter BNE = 8'hD0; // 3+,2 / 3,1
parameter BEQ = 8'hF0; // 3+,2 / 3,1

//time delay - opcode filler
parameter NOP  = 8'hEA; // 2 / 1

//flag set/clear
parameter CLC  = 8'h18; // 2 / 1
parameter SEC  = 8'h38; // 2 / 1
parameter CLI  = 8'h58; // 2 / 1
parameter SEI  = 8'h78; // 2 / 1
parameter CLV  = 8'hB8; // 2 / 1
parameter CLD  = 8'hD8; // 2 / 1
parameter SED  = 8'hF8; // 2 / 1

//accumulatior unary
parameter ASLA = 8'h0A; // 2 / 1
parameter ROLA = 8'h2A; // 2 / 1
parameter LSRA = 8'h4A; // 2 / 1
parameter RORA = 8'h6A; // 2 / 1

//y unary
parameter DEY  = 8'h88; // 2 / 1
parameter INY  = 8'hC8; // 2 / 1

//x unary
parameter DEX  = 8'hCA; // 2 / 1
parameter INX  = 8'hE8; // 2 / 1

//internal transfers
parameter TXA  = 8'h8A; // 2 / 1
parameter TAX  = 8'hAA; // 2 / 1
parameter TYA  = 8'h98; // 2 / 1
parameter TAY  = 8'hA8; // 2 / 1
parameter TXS  = 8'h9A; // 2 / 1
parameter TSX  = 8'hBA; // 2 / 1

//stack operations
parameter PHP = 8'h08; // 3 / 1
parameter PHA = 8'h48; // 3 / 1
parameter PLP = 8'h28; // 4 / 1
parameter PLA = 8'h68; // 4 / 2

//immediate - register
parameter ANDA_I = 8'h29; // 2 / 1
parameter EORA_I = 8'h49; // 2 / 1
parameter ORA_I  = 8'h09; // 2 / 1
parameter LDY_I  = 8'hA0; // 2 / 1
parameter LDX_I  = 8'hA2; // 2 / 1
parameter LDA_I  = 8'hA9; // 2 / 1
parameter CMPA_I = 8'hC9; // 2 / 1
parameter ADCA_I = 8'h69; // 2 / 1+
parameter SBCA_I = 8'hE9; // 2 / 1+
parameter CPY_I  = 8'hC0; // 2 / 1
parameter CPX_I  = 8'hE0; // 2 / 1

//memory-register operations

//zero page
parameter ORA_Z  = 8'h05; // 3 / 2
parameter BITA_Z = 8'h24; // 3 / 2
parameter ANDA_Z = 8'h25; // 3 / 2
parameter EORA_Z = 8'h45; // 3 / 2
parameter LDY_Z  = 8'hA4; // 3 / 2
parameter LDA_Z  = 8'hA5; // 3 / 2
parameter LDX_Z  = 8'hA6; // 3 / 2
parameter CMPA_Z = 8'hC5; // 3 / 2
parameter ADCA_Z = 8'h65; // 3 / 2+
parameter SBCA_Z = 8'hE5; // 3 / 2+
parameter CPY_Z  = 8'hC4; // 3 / 2
parameter CPX_Z  = 8'hE4; // 3 / 2

//zero page indexed
parameter ORA_ZX  = 8'h15; // 4 / 2
parameter ANDA_ZX = 8'h35; // 4 / 2
parameter EORA_ZX = 8'h55; // 4 / 2
parameter LDY_ZX  = 8'hB4; // 4 / 2
parameter LDA_ZX  = 8'hB5; // 4 / 2
parameter CMPA_ZX = 8'hD5; // 4 / 2
parameter ADCA_ZX = 8'h75; // 4 / 2+
parameter SBCA_ZX = 8'hF5; // 4 / 2+

parameter LDX_ZY = 8'hB6; // 4 / 2

//indexed indirect
parameter ORA_IX  = 8'h01; // 6 / 4,3* (>0x01FF, <0x0200) actual address
parameter ANDA_IX = 8'h21; // 6 / 4,3*
parameter EORA_IX = 8'h41; // 6 / 4,3*
parameter LDA_IX  = 8'hA1; // 6 / 4,3*
parameter CMPA_IX = 8'hC1; // 6 / 4,3*
parameter ADCA_IX = 8'h61; // 6 / 4+,3+*
parameter SBCA_IX = 8'hE1; // 6 / 4+,3+*

//indirect indexed
parameter ORA_IY  = 8'h11; // 5+ / 5,4*
parameter ANDA_IY = 8'h31; // 5+ / 5,4*
parameter EORA_IY = 8'h51; // 5+ / 5,4*
parameter LDA_IY  = 8'hB1; // 5+ / 5,4*
parameter CMPA_IY = 8'hD1; // 5+ / 5,4*
parameter ADCA_IY = 8'h71; // 5+ / 5+,4+*
parameter SBCA_IY = 8'hF1; // 5+ / 5+,4+*

//absolute
parameter ORA_A  = 8'h0D; // 4 / 3,2*
parameter BITA_A = 8'h2C; // 4 / 3,2*
parameter ANDA_A = 8'h2D; // 4 / 3,2*
parameter EORA_A = 8'h4D; // 4 / 3,2*
parameter LDY_A  = 8'hAC; // 4 / 3,2*
parameter LDA_A  = 8'hAD; // 4 / 3,2*
parameter LDX_A  = 8'hAE; // 4 / 3,2*
parameter CMPA_A = 8'hCD; // 4 / 3,2*
parameter ADCA_A = 8'h6D; // 4 / 3+,2+*
parameter SBCA_A = 8'hED; // 4 / 3+,2+*
parameter CPY_A  = 8'hCC; // 4 / 3,2*
parameter CPX_A  = 8'hEC; // 4 / 3,2*

//absolute indexed
parameter ORA_AX  = 8'h1D; // 4+ / 3,2*
parameter ANDA_AX = 8'h3D; // 4+ / 3,2*
parameter EORA_AX = 8'h5D; // 4+ / 3,2*
parameter LDY_AX  = 8'hBC; // 4+ / 3,2*
parameter LDA_AX  = 8'hBD; // 4+ / 3,2*
parameter CMPA_AX = 8'hDD; // 4+ / 3,2*
parameter ADCA_AX = 8'h7D; // 4+ / 3+,2+*
parameter SBCA_AX = 8'hFD; // 4+ / 3+,2+*

parameter ORA_AY  = 8'h19; // 4+ / 3,2*
parameter ANDA_AY = 8'h39; // 4+ / 3,2*
parameter EORA_AY = 8'h59; // 4+ / 3,2*
parameter LDA_AY  = 8'hB9; // 4+ / 3,2*
parameter LDX_AY  = 8'hBE; // 4+ / 3,2*
parameter CMPA_AY = 8'hD9; // 4+ / 3,2*
parameter ADCA_AY = 8'h79; // 4+ / 3+,2+*
parameter SBCA_AY = 8'hF9; // 4+ / 3+,2+*

//memory operations
parameter ASL_Z = 8'h06; // 5 / 2
parameter ROL_Z = 8'h26; // 5 / 2
parameter LSR_Z = 8'h46; // 5 / 2
parameter ROR_Z = 8'h66; // 5 / 2
parameter DEC_Z = 8'hC6; // 5 / 2
parameter INC_Z = 8'hE6; // 5 / 2

parameter ASL_ZX = 8'h16; // 6 / 2
parameter ROL_ZX = 8'h36; // 6 / 2
parameter LSR_ZX = 8'h56; // 6 / 2
parameter ROR_ZX = 8'h76; // 6 / 2
parameter DEC_ZX = 8'hD6; // 6 / 2
parameter INC_ZX = 8'hF6; // 6 / 2

//these slowed down by one cycle going from harvard to v.n. for external zp/stack address range updates only
parameter ASL_A = 8'h0E; // 6 / 3
parameter ROL_A = 8'h2E; // 6 / 3
parameter LSR_A = 8'h4E; // 6 / 3
parameter ROR_A = 8'h6E; // 6 / 3
parameter DEC_A = 8'hCE; // 6 / 3
parameter INC_A = 8'hEE; // 6 / 3

parameter ASL_AX = 8'h1E; // 7 / 3
parameter ROL_AX = 8'h3E; // 7 / 3
parameter LSR_AX = 8'h5E; // 7 / 3
parameter ROR_AX = 8'h7E; // 7 / 3
parameter DEC_AX = 8'hDE; // 7 / 3
parameter INC_AX = 8'hFE; // 7 / 3

//store operations (register-memory)
parameter STA_IX  = 8'h81; // 5 / 2
parameter STA_IY  = 8'h91; // 6 / 3

//these slowed down by one cycle going from harvard to v.n. for external zp/stack address range updates only
parameter STY_Z   = 8'h84; // 3 / 2
parameter STA_Z   = 8'h85; // 3 / 2
parameter STX_Z   = 8'h86; // 3 / 2
parameter STY_ZX  = 8'h94; // 3 / 2
parameter STA_ZX  = 8'h95; // 3 / 2
parameter STX_ZY  = 8'h96; // 3 / 2

//only these slowed down by one cycle going from harvard to v.n.
parameter STY_A   = 8'h8C; // 3 / 2
parameter STA_A   = 8'h8D; // 3 / 2
parameter STX_A   = 8'h8E; // 3 / 2
parameter STA_AX  = 8'h9D; // 3 / 2
parameter STA_AY  = 8'h99; // 3 / 2

reg two_byte;
reg asla, rola, lsra, rora, dex, dey, inx, iny;
reg txa, tya, txs, tay, tax, tsx, pha, pla;
reg clc, sec, cli, sei, cld, sed, clv, php, plp;
reg ora_i, anda_i, eora_i, lda_i, cmpa_i, adca_i, sbca_i;
reg ora_z, anda_z, eora_z, lda_z, cmpa_z, adca_z, sbca_z, bita_z;
reg ora_zx, anda_zx, eora_zx, lda_zx, cmpa_zx, adca_zx, sbca_zx;
reg sta_ix, ora_ix, anda_ix, eora_ix, lda_ix, cmpa_ix, adca_ix, sbca_ix;
reg sta_iy, ora_iy, anda_iy, eora_iy, lda_iy, cmpa_iy, adca_iy, sbca_iy;
reg ora_a, anda_a, eora_a, lda_a, cmpa_a, bita_a, adca_a, sbca_a;
reg ora_ax, anda_ax, eora_ax, ldy_ax, lda_ax, cmpa_ax, adca_ax, sbca_ax;
reg ora_ay, anda_ay, eora_ay, lda_ay, ldx_ay, cmpa_ay, adca_ay, sbca_ay;
reg asl_z, rol_z, lsr_z, ror_z, dec_z, inc_z;
reg asl_zx, rol_zx, lsr_zx, ror_zx, dec_zx, inc_zx;
reg asl_a, rol_a, lsr_a, ror_a, dec_a, inc_a;
reg asl_ax, rol_ax, lsr_ax, ror_ax, dec_ax, inc_ax;
reg stx_z, ldx_z, cpx_z, stx_zy, ldx_zy, stx_a, ldx_a, cpx_a, ldx_i, cpx_i;
reg sty_z, ldy_z, cpy_z, sty_zx, ldy_zx, sty_a, ldy_a, cpy_a, ldy_i, cpy_i;
reg bpl, bmi, bvc, bvs, bcc, bcs, bne, beq;
reg brk, rti, rts, jmp_a, jmp_i, jsr_a;
reg ix, iy, st_zxy, st_axy, zx, zy, ax, ay, mem_op, reg_op;
//reg nop, sta_z, sta_zx, sta_a, sta_ax, sta_ay;
//reg [7:0] irr;

always@(posedge clk)
  begin

    brk   <= L;
    rti   <= L;
    jsr_a <= L;
    rts   <= L;
    jmp_a <= L;
    jmp_i <= L;

    bpl <= L;
    bmi <= L;
    bvc <= L;
    bvs <= L;
    bcc <= L;
    bcs <= L;
    bne <= L;
    beq <= L;

    //nop  <= L;
    asla <= L;
    clc  <= L;
    rola <= L;
    sec  <= L;
    lsra <= L;
    cli  <= L;
    rora <= L;
    sei  <= L;
    dey  <= L;
    txa  <= L;
    tya  <= L;
    txs  <= L;
    tay  <= L;
    tax  <= L;
    clv  <= L;
    tsx  <= L;
    iny  <= L;
    dex  <= L;
    cld  <= L;
    inx  <= L;
    sed  <= L;

    php <= L;
    pha <= L;
    plp <= L;
    pla <= L;

    anda_i <= L;
    eora_i <= L;
    ora_i  <= L;
    ldy_i  <= L;
    ldx_i  <= L;
    lda_i  <= L;
    cmpa_i <= L;
    adca_i <= L;
    sbca_i <= L;
    cpy_i  <= L;
    cpx_i  <= L;

    sta_ix  <= L;
    ora_ix  <= L;
    anda_ix <= L;
    eora_ix <= L;
    lda_ix  <= L;
    cmpa_ix <= L;
    adca_ix <= L;
    sbca_ix <= L;
    ix <= L;
    
    sta_iy  <= L;
    ora_iy  <= L;
    anda_iy <= L;
    eora_iy <= L;
    lda_iy  <= L;
    cmpa_iy <= L;
    adca_iy <= L;
    sbca_iy <= L;
    iy <= L;
    
    sty_z  <= L;
    //sta_z  <= L;
    stx_z  <= L;
    ora_z  <= L;
    bita_z <= L;
    anda_z <= L;
    eora_z <= L;
    ldy_z  <= L;
    lda_z  <= L;
    ldx_z  <= L;
    cmpa_z <= L;
    adca_z <= L;
    sbca_z <= L;
    cpy_z  <= L;
    cpx_z  <= L;

    sty_zx  <= L;
    //sta_zx  <= L;
    ora_zx  <= L;
    anda_zx <= L;
    eora_zx <= L;
    ldy_zx  <= L;
    lda_zx  <= L;
    cmpa_zx <= L;
    adca_zx <= L;
    sbca_zx <= L;

    stx_zy <= L;
    ldx_zy <= L;
    zy <= L;

    sty_a  <= L;
    //sta_a  <= L;
    stx_a  <= L;
    ora_a  <= L;
    bita_a <= L;
    anda_a <= L;
    eora_a <= L;
    ldy_a  <= L;
    lda_a  <= L;
    ldx_a  <= L;
    cmpa_a <= L;
    adca_a <= L;
    sbca_a <= L;
    cpy_a  <= L;
    cpx_a  <= L;

    //sta_ax  <= L;
    ora_ax  <= L;
    anda_ax <= L;
    eora_ax <= L;
    ldy_ax  <= L;
    lda_ax  <= L;
    cmpa_ax <= L;
    adca_ax <= L;
    sbca_ax <= L;

    //sta_ay  <= L;
    ora_ay  <= L;
    anda_ay <= L;
    eora_ay <= L;
    lda_ay  <= L;
    ldx_ay  <= L;
    cmpa_ay <= L;
    adca_ay <= L;
    sbca_ay <= L;
    ay <= L;

    asl_z <= L;
    rol_z <= L;
    lsr_z <= L;
    ror_z <= L;
    dec_z <= L;
    inc_z <= L;

    asl_zx <= L;
    rol_zx <= L;
    lsr_zx <= L;
    ror_zx <= L;
    dec_zx <= L;
    inc_zx <= L;
    zx <= L;

    asl_a <= L;
    rol_a <= L;
    lsr_a <= L;
    ror_a <= L;
    dec_a <= L;
    inc_a <= L;

    asl_ax <= L;
    rol_ax <= L;
    lsr_ax <= L;
    ror_ax <= L;
    dec_ax <= L;
    inc_ax <= L;
    ax <= L;

    st_zxy <= L;
    st_axy <= L;
    
    mem_op <= L;
    reg_op <= L;
    
    //irr <= ir;
    
    case(ir)
        BRK:
          begin
            brk <= H;
            two_byte <= L;
          end
        RTI:
          begin
            rti <= H;
            two_byte <= L;
          end
        JSR_A:
          begin
            jsr_a <= H;
            two_byte <= L;
          end
        RTS:
          begin
            rts <= H;
            two_byte <= L;
          end
        JMP_A:
          begin
            jmp_a <= H;
            two_byte <= L;
          end
        JMP_I:
          begin
            jmp_i <= H;
            two_byte <= L;
          end

        BPL:
          begin
            bpl <= H;
            two_byte <= H;
          end
        BMI:
          begin
            bmi <= H;
            two_byte <= H;
          end
        BVC:
          begin
            bvc <= H;
            two_byte <= H;
          end
        BVS:
          begin
            bvs <= H;
            two_byte <= H;
          end
        BCC:
          begin
            bcc <= H;
            two_byte <= H;
          end
        BCS:
          begin
            bcs <= H;
            two_byte <= H;
          end
        BNE:
          begin
            bne <= H;
            two_byte <= H;
          end
        BEQ:
          begin
            beq <= H;
            two_byte <= H;
          end


        NOP:
          begin
            //nop <= H;
            two_byte <= L;
          end
        ASLA:
          begin
            asla <= H;
            two_byte <= L;
          end
        CLC:
          begin
            clc <= H;
            two_byte <= L;
          end
        ROLA:
          begin
            rola <= H;
            two_byte <= L;
          end
        SEC:
          begin
            sec <= H;
            two_byte <= L;
          end
        LSRA:
          begin
            lsra <= H;
            two_byte <= L;
          end
        CLI:
          begin
            cli <= H;
            two_byte <= L;
          end
        RORA:
          begin
            rora <= H;
            two_byte <= L;
          end
        SEI:
          begin
            sei <= H;
            two_byte <= L;
          end
        DEY:
          begin
            dey <= H;
            two_byte <= L;
          end
        TXA:
          begin
            txa <= H;
            two_byte <= L;
          end
        TYA:
          begin
            tya <= H;
            two_byte <= L;
          end
        TXS:
          begin
            txs <= H;
            two_byte <= L;
          end
        TAY:
          begin
            tay <= H;
            two_byte <= L;
          end
        TAX:
          begin
            tax <= H;
            two_byte <= L;
          end
        CLV:
          begin
            clv  <= H;
            two_byte <= L;
          end
        TSX:
          begin
            tsx <= H;
            two_byte <= L;
          end
        INY:
          begin
            iny <= H;
            two_byte <= L;
          end
        DEX:
          begin
            dex <= H;
            two_byte <= L;
          end
        CLD:
          begin
            cld <= H;
            two_byte <= L;
          end
        INX:
          begin
            inx <= H;
            two_byte <= L;
          end
        SED:
          begin
            sed <= H;
            two_byte <= L;
          end

        PHP:
          begin
            php <= H;
            two_byte <= L;
          end
        PHA:
          begin
            pha <= H;
            two_byte <= L;
          end
        PLP:
          begin
            plp <= H;
            two_byte <= L;
          end
        PLA:
          begin
            pla <= H;
            two_byte <= L;
          end


        ANDA_I:
          begin
            anda_i <= H;
            two_byte <= H;
          end
        EORA_I:
          begin
            eora_i <= H;
            two_byte <= H;
          end
         ORA_I:
          begin
            ora_i  <= H;
            two_byte <= H;
          end
        LDY_I:
          begin
            ldy_i  <= H;
            two_byte <= H;
          end
        LDX_I:
          begin
            ldx_i  <= H;
            two_byte <= H;
          end
        LDA_I:
          begin
            lda_i  <= H;
            two_byte <= H;
          end
        CMPA_I:
          begin
            cmpa_i <= H;
            two_byte <= H;
          end
        ADCA_I:
          begin
            adca_i <= H;
            two_byte <= H;
          end
        SBCA_I:
          begin
            sbca_i <= H;
            two_byte <= H;
          end
        CPY_I:
          begin
            cpy_i  <= H;
            two_byte <= H;
          end
        CPX_I:
          begin
            cpx_i  <= H;
            two_byte <= H;
          end


        STA_IX:
          begin
            sta_ix  <= H;
            two_byte <= H;
            ix <= H;
          end
        ORA_IX:
          begin
            ora_ix  <= H;
            two_byte <= H;
            ix <= H;
          end
        ANDA_IX:
          begin
            anda_ix <= H;
            two_byte <= H;
            ix <= H;
          end
        EORA_IX:
          begin
            eora_ix <= H;
            two_byte <= H;
            ix <= H;
          end
        LDA_IX:
          begin
            lda_ix  <= H;
            two_byte <= H;
            ix <= H;
          end
        CMPA_IX:
          begin
            cmpa_ix <= H;
            two_byte <= H;
            ix <= H;
          end
        ADCA_IX:
          begin
            adca_ix <= H;
            two_byte <= H;
            ix <= H;
          end
        SBCA_IX:
          begin
            sbca_ix <= H;
            two_byte <= H;
            ix <= H;
          end

        STA_IY:
          begin
            sta_iy  <= H;
            two_byte <= H;
            iy <= H;
          end
        ORA_IY:
          begin
            ora_iy  <= H;
            two_byte <= H;
            iy <= H;
          end
        ANDA_IY:
          begin
            anda_iy <= H;
            two_byte <= H;
            iy <= H;
          end
        EORA_IY:
          begin
            eora_iy <= H;
            two_byte <= H;
            iy <= H;
          end
        LDA_IY:
          begin
            lda_iy  <= H;
            two_byte <= H;
            iy <= H;
          end
        CMPA_IY:
          begin
            cmpa_iy <= H;
            two_byte <= H;
            iy <= H;
          end
        ADCA_IY:
          begin
            adca_iy <= H;
            two_byte <= H;
            iy <= H;
          end
        SBCA_IY:
          begin
            sbca_iy <= H;
            two_byte <= H;
            iy <= H;
          end

        STY_Z:
          begin
            sty_z  <= H;
            two_byte <= H;
            st_zxy <= H;
          end
        STA_Z:
          begin
            //sta_z  <= H;
            two_byte <= H;
            st_zxy <= H;
          end
        STX_Z:
          begin
            stx_z  <= H;
            two_byte <= H;
            st_zxy <= H;
          end
        ORA_Z:
          begin
            ora_z  <= H;
            two_byte <= H;
            reg_op <= H;
          end
        BITA_Z:
          begin
            bita_z <= H;
            two_byte <= H;
            reg_op <= H;
          end
        ANDA_Z:
          begin
            anda_z <= H;
            two_byte <= H;
            reg_op <= H;
          end
        EORA_Z:
          begin
            eora_z <= H;
            two_byte <= H;
            reg_op <= H;
          end
        LDY_Z:
          begin
            ldy_z  <= H;
            two_byte <= H;
            reg_op <= H;
          end
        LDA_Z:
          begin
            lda_z  <= H;
            two_byte <= H;
            reg_op <= H;
          end
        LDX_Z:
          begin
            ldx_z  <= H;
            two_byte <= H;
            reg_op <= H;
          end
        CMPA_Z:
          begin
            cmpa_z <= H;
            two_byte <= H;
            reg_op <= H;
          end
        ADCA_Z:
          begin
            adca_z <= H;
            two_byte <= H;
            reg_op <= H;
          end
        SBCA_Z:
          begin
            sbca_z <= H;
            two_byte <= H;
            reg_op <= H;
          end
        CPY_Z:
          begin
            cpy_z  <= H;
            two_byte <= H;
            reg_op <= H;
          end
        CPX_Z:
          begin
            cpx_z  <= H;
            two_byte <= H;
            reg_op <= H;
          end


        STY_ZX:
          begin
            sty_zx  <= H;
            two_byte <= H;
            st_zxy <= H;
            zx <= H;
          end
        STA_ZX:
          begin
            //sta_zx  <= H;
            two_byte <= H;
            st_zxy <= H;
            zx <= H;
          end
        ORA_ZX:
          begin
            ora_zx  <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end
        ANDA_ZX:
          begin
            anda_zx <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end
        EORA_ZX:
          begin
            eora_zx <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end
        LDY_ZX:
          begin
            ldy_zx  <= H;
            two_byte <= H;
             zx <= H;
            reg_op <= H;
         end
        LDA_ZX:
          begin
            lda_zx  <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end
        CMPA_ZX:
          begin
            cmpa_zx <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end
        ADCA_ZX:
          begin
            adca_zx <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end
        SBCA_ZX:
          begin
            sbca_zx <= H;
            two_byte <= H;
            zx <= H;
            reg_op <= H;
          end


        STX_ZY:
          begin
            stx_zy <= H;
            two_byte <= H;
            st_zxy <= H;
            zy <= H;
          end
        LDX_ZY:
          begin
            ldx_zy <= H;
            two_byte <= H;
            zy <= H;
            reg_op <= H;
          end


        STY_A:
          begin
            sty_a  <= H;
            two_byte <= L;
            st_axy <= H;
          end
        STA_A:
          begin
            //sta_a  <= H;
            two_byte <= L;
            st_axy <= H;
          end
        STX_A:
          begin
            stx_a  <= H;
            two_byte <= L;
            st_axy <= H;
          end
        ORA_A:
          begin
            ora_a  <= H;
            two_byte <= L;
          end
        BITA_A:
          begin
            bita_a <= H;
            two_byte <= L;
          end
        ANDA_A:
          begin
            anda_a <= H;
            two_byte <= L;
          end
        EORA_A:
          begin
            eora_a <= H;
            two_byte <= L;
          end
        LDY_A:
          begin
            ldy_a  <= H;
            two_byte <= L;
          end
        LDA_A:
          begin
            lda_a  <= H;
            two_byte <= L;
          end
        LDX_A:
          begin
            ldx_a  <= H;
            two_byte <= L;
          end
        CMPA_A:
          begin
            cmpa_a <= H;
            two_byte <= L;
          end
        ADCA_A:
          begin
            adca_a <= H;
            two_byte <= L;
          end
        SBCA_A:
          begin
            sbca_a <= H;
            two_byte <= L;
          end
        CPY_A:
          begin
            cpy_a  <= H;
            two_byte <= L;
          end
        CPX_A:
          begin
            cpx_a  <= H;
            two_byte <= L;
          end


        STA_AX:
          begin
            //sta_ax  <= H;
            two_byte <= L;
            st_axy <= H;
            ax <= H;
          end
        ORA_AX:
          begin
            ora_ax  <= H;
            two_byte <= L;
            ax <= H;
          end
        ANDA_AX:
          begin
            anda_ax <= H;
            two_byte <= L;
            ax <= H;
          end
        EORA_AX:
          begin
            eora_ax <= H;
            two_byte <= L;
            ax <= H;
          end
        LDY_AX:
          begin
            ldy_ax  <= H;
            two_byte <= L;
            ax <= H;
          end
        LDA_AX:
          begin
            lda_ax  <= H;
            two_byte <= L;
            ax <= H;
          end
        CMPA_AX:
          begin
            cmpa_ax <= H;
            two_byte <= L;
            ax <= H;
          end
        ADCA_AX:
          begin
            adca_ax <= H;
            two_byte <= L;
            ax <= H;
          end
        SBCA_AX:
          begin
            sbca_ax <= H;
            two_byte <= L;
            ax <= H;
          end


        STA_AY:
          begin
            //sta_ay  <= H;
            two_byte <= L;
            st_axy <= H;
            ay <= H;
          end
        ORA_AY:
          begin
            ora_ay  <= H;
            two_byte <= L;
            ay <= H;
          end
        ANDA_AY:
          begin
            anda_ay <= H;
            two_byte <= L;
            ay <= H;
          end
        EORA_AY:
          begin
            eora_ay <= H;
            two_byte <= L;
            ay <= H;
          end
        LDA_AY:
          begin
            lda_ay  <= H;
            two_byte <= L;
            ay <= H;
          end
        LDX_AY:
          begin
            ldx_ay  <= H;
            two_byte <= L;
            ay <= H;
          end
        CMPA_AY:
          begin
            cmpa_ay <= H;
            two_byte <= L;
            ay <= H;
          end
        ADCA_AY:
          begin
            adca_ay <= H;
            two_byte <= L;
            ay <= H;
          end
        SBCA_AY:
          begin
            sbca_ay <= H;
            two_byte <= L;
            ay <= H;
          end


        ASL_Z:
          begin
            asl_z <= H;
            two_byte <= H;
            mem_op <= H;
         end
        ROL_Z:
          begin
            rol_z <= H;
            two_byte <= H;
            mem_op <= H;
          end
        LSR_Z:
          begin
            lsr_z <= H;
            two_byte <= H;
            mem_op <= H;
           end
        ROR_Z:
          begin
            ror_z <= H;
            two_byte <= H;
            mem_op <= H;
          end
        DEC_Z:
          begin
            dec_z <= H;
            two_byte <= H;
            mem_op <= H;
          end
        INC_Z:
          begin
            inc_z <= H;
            two_byte <= H;
            mem_op <= H;
          end


        ASL_ZX:
          begin
            asl_zx <= H;
            two_byte <= H;
            zx <= H;
            mem_op <= H;
          end
        ROL_ZX:
          begin
            rol_zx <= H;
            two_byte <= H;
            zx <= H;
            mem_op <= H;
          end
        LSR_ZX:
          begin
            lsr_zx <= H;
            two_byte <= H;
            zx <= H;
            mem_op <= H;
          end
        ROR_ZX:
          begin
            ror_zx <= H;
            two_byte <= H;
            zx <= H;
            mem_op <= H;
          end
        DEC_ZX:
          begin
            dec_zx <= H;
            two_byte <= H;
            zx <= H;
            mem_op <= H;
          end
        INC_ZX:
          begin
            inc_zx <= H;
            two_byte <= H;
            zx <= H;
            mem_op <= H;
          end


        ASL_A:
          begin
            asl_a <= H;
            two_byte <= L;
            mem_op <= H;
          end
        ROL_A:
          begin
            rol_a <= H;
            two_byte <= L;
            mem_op <= H;
          end
        LSR_A:
          begin
            lsr_a <= H;
            two_byte <= L;
            mem_op <= H;
          end
        ROR_A:
          begin
            ror_a <= H;
            two_byte <= L;
            mem_op <= H;
          end
        DEC_A:
          begin
            dec_a <= H;
            two_byte <= L;
            mem_op <= H;
          end
        INC_A:
          begin
            inc_a <= H;
            two_byte <= L;
            mem_op <= H;
          end


        ASL_AX:
          begin
            asl_ax <= H;
            two_byte <= L;
            ax <= H;
            mem_op <= H;
          end
        ROL_AX:
          begin
            rol_ax <= H;
            two_byte <= L;
            ax <= H;
            mem_op <= H;
          end
        LSR_AX:
          begin
            lsr_ax <= H;
            two_byte <= L;
            ax <= H;
            mem_op <= H;
          end
        ROR_AX:
          begin
            ror_ax <= H;
            two_byte <= L;
            ax <= H;
            mem_op <= H;
          end
        DEC_AX:
          begin
            dec_ax <= H;
            two_byte <= L;
            ax <= H;
            mem_op <= H;
          end
        INC_AX:
          begin
            inc_ax <= H;
            two_byte <= L;
            ax <= H;
            mem_op <= H;
          end

        default:
          begin
            two_byte <= L;
          end

      endcase
  end
  
reg one_byte_a;
reg three_byte_a;

always@(*)
  begin
    case(ir)

      BRK,
      NOP,
      CLV,
      CLC, SEC,
      CLI, SEI,
      CLD, SED,
      RTI, RTS,
      ASLA, LSRA,
      ROLA, RORA,
      TAY, TYA,
      TAX, TXA,
      TSX, TXS,
      INY, DEY,
      INX, DEX,
      PHP, PLP,
      PHA, PLA:
        begin
          one_byte_a = H;
        end

        default: one_byte_a = L;

    endcase
  end

always@(*)
  begin
    case(ir)
     
      JSR_A, JMP_A, JMP_I,
      STY_A, LDY_A, CPY_A,
      STX_A, LDX_A, CPX_A,
      STA_A,  ORA_A,  ANDA_A,  EORA_A,  LDA_A,  CMPA_A,  ADCA_A,  SBCA_A,  BITA_A,
      STA_AX, ORA_AX, ANDA_A,  EORA_AX, LDA_AX, CMPA_AX, ADCA_AX, SBCA_AX,
      STA_AY, ORA_AY, ANDA_AY, EORA_AY, LDA_AY, CMPA_AY, ADCA_AY, SBCA_AY,
      LDY_AX,
      LDX_AY,
      ASL_A,  ROL_A,  LSR_A,  ROR_A,  DEC_A,  INC_A,
      ASL_AX, ROL_AX, LSR_AX, ROR_AX, DEC_AX, INC_AX:
        begin
          three_byte_a = H;
        end

      default: three_byte_a = L;

    endcase
  end

endmodule
