//****************************************************************************
// C64 VGA timing (320x200).
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

module vgaTiming( mclk, vclk, HS1, VS1, hActive, vActive, hCounter, vCounter );

input mclk;
input vclk;

output [8:0] hCounter;
output [9:0] vCounter;
output HS1, VS1;
output hActive, vActive;

parameter H = 1'b1;
parameter L = 1'b0;

reg [8:0] hCounter = 0;
reg [9:0] vCounter = 0;

reg HS, HS1, HS2, HS3, HS4;
reg VS, VS1, VS2, VS3, VS4;

reg hActive;
reg vActive;

always @(posedge mclk)
  begin
    if(vclk)
      begin    
        //only update video when this signal is high
        //the hCounter just rolls from 511 to 0 after each scan line
        hCounter <= hCounter + H;

        if(hCounter == 511)
          begin
            HS <= L; //hsync starts at count 0
  
            if(vCounter == 524) //525 vertical scan lines
              begin
                vCounter <= 0;
                VS <= L;
              end
            else
              begin
                vCounter <= vCounter + H;
              end

            if(vCounter == 1) VS <= H;
            
            if(vCounter == 34) vActive <= H;
            else if(vCounter == 514) vActive <= L;
            
          end
        else
          begin
            if(hCounter == 61) HS <= H;
            
            if(hCounter == 92) hActive <= H;
            else if(hCounter == 501) hActive <= L;
          end

        //delay these to match up with actual video processing delays
        HS4 <= HS;
        VS4 <= VS;
        HS3 <= HS4;
        VS3 <= VS4;
        HS2 <= HS3;
        VS2 <= VS3;
        HS1 <= HS2;
        VS1 <= VS2;
      end
  end //end always

endmodule

